<?php
namespace App\Http\Controllers\adminpanel;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\How_hear_about;
use Validator;
use Illuminate\Pagination\Paginator;
use App\Traits\UseAutoIncrementID;
use Illuminate\Support\Facades\DB;

class UserController extends Controller 
{
    private $pagination = 20;

    public function manage() {
        $data = User::query()->paginate($this->pagination);
        return view('adminpanel.manageuser', compact('data'));
    }

    public function search(Request $request) {
        $input = $request->all();
        $qry = User::query(); 
        if(trim($input["search"])!="") {
            $search = $input["search"];
            $qry->where([
                ["username", "like", "%{$search}%"],
            ]);
            $qry->orwhere([
                ["email", "like", "%{$search}%"],
            ]);
        }
        $data = $qry->paginate($this->pagination);
        $data->appends($input);
        return view('adminpanel.manageuser', compact('data'));
    }

    public function add() {
        $how_hear_abouts = How_hear_about::where("is_active", "=", 1)->pluck('name', 'id');
        $data = array('type'=>'add', 'how_hear_abouts'=>$how_hear_abouts);
        return view('adminpanel.adduser', compact('data'));
    }

    public function save(Request $request) {
        $input = $request->all();
        $validator = Validator::make( $input, $this->getRules('Add', $input), $this->messages());
        if ($validator->fails()) { 
            $how_hear_abouts = How_hear_about::where("is_active", "=", 1)->pluck('name', 'id');
            $data = array('type'=>'add', 'input'=>$input, 'how_hear_abouts'=>$how_hear_abouts, 'error'=>$validator->messages());
            return view('adminpanel.adduser', compact('data'));
            exit();            
        }

        $input['age'] = intval($input['age']);
        $input['score'] = intval($input['score']);
        $input['password'] = bcrypt($input['password']);
        $input['register_type'] = 'email';
        if (isset($input['status'])) 
        {
            $input["status"] = 1;
        }
        else
        {
            $input["status"] = 0;
        }

        if(isset($input["profilePicture"])) 
        {
            $imagePath = $input["profilePicture"];
            $filename = rand(0000,9999).$imagePath->getClientOriginalName();
            $extension  = pathinfo($imagePath->getClientOriginalName(), PATHINFO_EXTENSION);
            $filename = rand(0000,9999)."profilePicture.".$extension;
            $upload_dir_path = public_path()."/uploads/profilePicture";
            $imagePath->move($upload_dir_path, $filename );  
            $input['profilePicture'] = $filename;          
        }

        
        /* Generate ID */
        $input['user_id'] = UseAutoIncrementID::getID('users');
       
        $user = User::create($input);
        if($user->user_id>0) 
        {
            return redirect()->route('adminpanel.user.manage')->with('success', 'Created successfully.');
        } else {
            return redirect()->route('adminpanel.user.add')->withErrors(['Error creating record. Please try again.']);
        }
    }

	public function edit($id) {
        $how_hear_abouts = How_hear_about::where("is_active", "=", 1)->pluck('name', 'id');
        $input = User::where('user_id', '=', intval($id))->get();
        $data = array('type'=>'edit', 'input'=>$input, 'how_hear_abouts'=>$how_hear_abouts);
	    return view('adminpanel.adduser', compact('data'));
	}

	public function update(Request $request) {
		$input = $request->all();
        $id = $input['id'];
        $validator = Validator::make( $input, $this->getRules('Edit', $input), $this->messages()); 
        
        if ($validator->fails()) { 
            $how_hear_abouts = How_hear_about::where("is_active", "=", "1")->pluck('name', 'id');
            $data = array('type'=>'Edit', 'input'=>$input, 'how_hear_abouts'=>$how_hear_abouts, 'error'=>$validator->messages());
            return view('adminpanel.adduser', compact('data'));
            exit();            
        }
        $update = array();
        $update["username"] = $input['username'];
        $update["phone"] = $input['phone'];
        $update['score'] = intval($input['score']);
        $update["email"] = $input['email'];
        if(isset($input['password']) && $input['password']!="") {
            $update["password"] = bcrypt($input['password']);
        }
        $update["age"] = $input['age'];
        $update["gender"] = $input['gender'];
        if (isset($input['status'])) 
        {
            $input["status"] = 1;
        }
        else
        {
            $input["status"] = 0;
        }
        if(isset($input["profilePicture"])) 
        {
            $imagePath = $input["profilePicture"];
            $filename = rand(0000,9999).$imagePath->getClientOriginalName();
            $extension  = pathinfo($imagePath->getClientOriginalName(), PATHINFO_EXTENSION);
            $filename = rand(0000,9999)."profilePicture.".$extension;
            $upload_dir_path = public_path()."/uploads/profilePicture";
            $imagePath->move($upload_dir_path, $filename );  
            $input['profilePicture'] = $filename;          
        }

        $user = User::where('user_id', '=', intval($id))->update($update);
        return redirect()->route('adminpanel.user.manage')->with('success', 'Updated successfully.');

	}

    public function delete($id) {
        User::where('user_id','=', intval($id))->delete();
        return redirect()->route('adminpanel.user.manage')->with('success', 'Deleted successfully.');
    }

    //$result = $this->removeuserimage($upload_dir_path, $id);
    private function removeuserimage($imagepath, $id) {
        $user = User::where('id', '=', $id)->get();
        if($user[0]->profile_picture!=null && $user[0]->profile_picture!="") {
            if(file_exists($imagepath.'\\'.$user[0]->profile_picture)) {
                unlink($imagepath.'\\'.$user[0]->profile_picture);
            }
        }
        return true;
    }

	private function getRules($type, $input) {
        $return = array();
        $return['username'] = 'required|max:30';
        $return['phone'] = 'required|max:20';
        if($type=="Edit") {
            $return['email'] = 'required|email|max:100';
            if(isset($input['password']) && $input["password"]!="") {
                $return['password'] = 'min:6|max:20';
            }
        } else {
            $return['email'] = 'required|email|unique:users,email|max:100';
            $return['password'] = 'required|min:6|max:20';
        }
        return $return;
    }

    private function messages() {
        return [
            'username.required'  => $this->getRequiredMessage('first name'),
            'username.max'  => $this->getGreaterMessage('first name', 30),
            'phone.required'  => $this->getRequiredMessage('phone no.'),
            'phone.max'  => $this->getGreaterMessage('phone no.', 20),
        ];
    }

    private function getRequiredMessage($string) {
        return 'The ' . $string . ' field is required.';
    }

    private function getGreaterMessage($string, $maxchar) {
        return 'The ' . $string . ' may not be greater than ' . $maxchar . ' characters.';
    }
}

?>